## load the packages
library(lme4)
library(nnet)
library(ggplot2)
library(gridExtra)
library(grid)
library(lattice)
library(BSDA)
library(boot)

path <- '~'
setwd(path)
getwd()

multiplot <- function(..., plotlist=NULL, file, cols=1, layout=NULL) {
    library(grid)
    
    # Make a list from the ... arguments and plotlist
    plots <- c(list(...), plotlist)
    
    numPlots = length(plots)
    
    # If layout is NULL, then use 'cols' to determine layout
    if (is.null(layout)) {
        # Make the panel
        # ncol: Number of columns of plots
        # nrow: Number of rows needed, calculated from # of cols
        layout <- matrix(seq(1, cols * ceiling(numPlots/cols)),
                         ncol = cols, nrow = ceiling(numPlots/cols))
    }
    
    if (numPlots==1) {
        print(plots[[1]])
        
    } else {
        # Set up the page
        grid.newpage()
        pushViewport(viewport(layout = grid.layout(nrow(layout), ncol(layout))))
        
        # Make each plot, in the correct location
        for (i in 1:numPlots) {
            # Get the i,j matrix positions of the regions that contain this subplot
            matchidx <- as.data.frame(which(layout == i, arr.ind = TRUE))
            
            print(plots[[i]], vp = viewport(layout.pos.row = matchidx$row,
                                            layout.pos.col = matchidx$col))
        }
    }
}

##############################DATA GENERATION#######################################
datgen=function(n,N,mu_Z_effects,hospital_generate_coefficient,a,b,p,a0,betax1,betax2)
{
  ##n=the number of patient
  ##N=the number of hospitals
  ##we generate two confounders:one continuous, one binary 
  ##(Disease progression)
  ##we assume the continuous confounder is following the 
  ##normal distribution with two parameter mean=a and sd=b
  ##and the Disease progression follow a bernulli distibution with
  ##parameter p
  ##parameter mu_Z_effect and sd_Z_effect are used to generate 
  ##hospital effect on outcome Y, which follow a normal distribution
  ##parameter a0 is the intercept that used to generate outcome Y
  ##parameter betax1,betax2 are confounder effect on outcome Y
  X1=rnorm(n,mean=a,sd=b)
  X2=rbinom(n,size=1,p)
  
  ##next step, we generate the hospital indicator Z conditional on confounder X
  ##by using the multinomial distribution
  ##we assume there are 5 hospitals
  X=as.matrix(data.frame(1,X1,X2))
  
  ##The number of hospital is N
  number_hospital=N
  
  #Calculation of denominator for probability calculation
  Denominator=1
  for(i in 1:(number_hospital-1))
  {
    Denominator=Denominator+exp(X%*%hospital_generate_coefficient[i,])
  }
  #Calculating the matrix of probabilities for these choices
  vProb=1/Denominator
  for (i in 1:(number_hospital-1))
  {
    vProb = cbind(vProb,exp(X%*%hospital_generate_coefficient[i,])/Denominator)
  }
  colMeans(vProb)
  #Assigning the value one to maximum probability 
  #and zero for rest to get the appropriate choices
  mChoices = t(apply(vProb, 1, rmultinom, n = 1, size = 1))
  colMeans(mChoices)
  
  # Value of Z and X together
  dfM = cbind.data.frame(Z = apply(mChoices, 1, function(x) which(x==1)), X[,2:3])
  Design_matrix=cbind(rep(1,n),mChoices,X1,X2)
  
  
  ##Finally, we generate the outcome Y
  ##coefficient that used to generate Y
  ##We use the normal distribution with mean=mu_Z_effect
  ##and sd=sd_Z_effect
  #Y_coe=c(a0,0,rnorm(number_hospital-1,mean=mu_Z_effect,sd=sd_Z_effect),
  #        betax1,betax2)
  Y_coe=c(a0,mu_Z_effects,betax1,betax2)
  ##continuous outcome Y
  Y_mean=Design_matrix%*%Y_coe
  Y_continuous=rlogis(n, location=Y_mean)
  ##binary outcome Y
  Y_prob=exp(Y_mean)/(1+exp(Y_mean))
  # Y_binary=rbinom(n, 1, Y_prob)
  Y_binary = as.numeric(Y_continuous >= 0.0)
  
  ##calculte the true variance explained by patient and hospital
  firstmoment_continuous=rep(0,n)
  firstmoment_binary=rep(0,n)
  secondmoment_continuous=rep(0,n)
  secondmoment_binary=rep(0,n)
  for (i in 1:number_hospital)
  {
    True_continuousY=Y_coe[1]+Y_coe[i+1]+betax1*X1+betax2*X2
    True_binaryY=exp(True_continuousY)/(1+exp(True_continuousY))
    
    firstmoment_continuous=firstmoment_continuous+True_continuousY*vProb[,i]
    firstmoment_binary=firstmoment_binary+True_binaryY*vProb[,i]
    
    secondmoment_continuous=secondmoment_continuous+True_continuousY^2*vProb[,i]
    secondmoment_binary=secondmoment_binary+True_binaryY^2*vProb[,i]
  }
  True_v_patient_continuous=firstmoment_continuous
  True_v_patient_binary=firstmoment_binary
  True_v_hospital_continuous=secondmoment_continuous-firstmoment_continuous^2
  True_v_hospital_binary=secondmoment_binary-firstmoment_binary^2
  
  dat=data.frame(Y_continuous,Y_binary,dfM,True_v_patient_continuous,
                 True_v_patient_binary,True_v_hospital_continuous,
                 True_v_hospital_binary)
  
  return (dat)
}

#######################################Analysis Continuous Outcome############################################
analysis_continuous=function(data_analysis)
{
  ##variance explained by residual by random effect model
  model_r_e=lmer(Y_continuous ~ X1 + X2+(1|Z), data_analysis,
                 control=lmerControl(optCtrl=list(maxfun=30000)))
  residual_variance_re=as.data.frame(VarCorr(model_r_e))[2,4]
  
  ##variance explained by residual by fixed effect model
  model_f_e=lm(Y_continuous ~as.factor(Z) + X1 + X2, data_analysis)
  residual_variance_fe=summary(model_f_e)$sigma^2
  
  ##variance explained by patient mixed-case
  ##method 1: fixed effect model regression
  model_l=lm(Y_continuous ~ X1+X2, data_analysis)
  patient_variance_fe=var(predict(model_l))
  
  ##method 2: PS method
  model_ps=multinom(Z ~ X1+X2, data = data_analysis,maxit=1000)
  predict_ps=fitted(model_ps)
  
  
  
  EY_ZX=NULL
  EY_ZX_FE=NULL
  VX=NULL
  VX_FE=NULL
  for (i in 1:max(data_analysis$Z))
  {
    Z_data=data_analysis
    Z_data$Z=rep(i,nrow(Z_data))
    ##by random effect model
    EY_ZX_temp=predict(model_r_e,Z_data)
    EY_ZX=cbind(EY_ZX,EY_ZX_temp)
    ##by fixed effect model
    EY_ZX_temp_fe=predict(model_f_e,Z_data)
    EY_ZX_FE=cbind(EY_ZX_FE,EY_ZX_temp_fe)
  }
  for (i in 1:nrow(EY_ZX))
  {
    VX[i]=sum(EY_ZX[i,]*predict_ps[i,])
    VX_FE[i]=sum(EY_ZX_FE[i,]*predict_ps[i,])
  }
  patient_variance_ps=var(VX)
  patient_variance_ps_fe=var(VX_FE)
  
  ##variance explained by patient hospital performance
  ##method 1
  ##random effect model
  hospital_variance_re=as.data.frame(VarCorr(model_r_e))[1,4]
  
  ##method 2
  ##PS method
  EX=NULL
  EX_FE=NULL
  
  for (i in 1:nrow(EY_ZX))
  {
    EX[i]=sum((EY_ZX[i,]^2)*predict_ps[i,])-(sum(EY_ZX[i,]*predict_ps[i,]))^2
    EX_FE[i]=sum((EY_ZX_FE[i,]^2)*predict_ps[i,])-(sum(EY_ZX_FE[i,]*predict_ps[i,]))^2
  }
  hospital_variance_ps=mean(EX)
  hospital_variance_ps_fe=mean(EX_FE)
  Variance_Y=var(data_analysis$Y_continuous)
  
  ##true_vairance_explained by hospital and patient
  True_patient_v_con=var(data_analysis$True_v_patient_continuous)
  True_hospital_v_con=mean(data_analysis$True_v_hospital_continuous)
  
  
  return(c(Variance_Y,residual_variance_fe,residual_variance_re,
           hospital_variance_re,hospital_variance_ps,hospital_variance_ps_fe,
           patient_variance_fe,patient_variance_ps,patient_variance_ps_fe,
           True_patient_v_con,True_hospital_v_con))
}

#######################################Analysis Binary Outcome############################################
analysis_binary=function(data_analysis)
{
  ##omit this part
  ##variance explained by residual by random effect model
  model_r_e=glmer(Y_binary ~ X1 + X2+(1|Z), data_analysis,family = binomial(link="logit"))
  #residual_variance_re=as.data.frame(VarCorr(model_r_e))[2,4]
  
  ##variance explained by residual by fixed effect model
  model_f_e=glm(Y_binary ~as.factor(Z) + X1 + X2, data_analysis,family = binomial(link="logit"))
  #residual_variance_fe=sigma(model_f_e)^2
  
  ##variance explained by patient mixed-case
  ##method 1: fixed effect model regression
  ##It is biased when outcome is binary
  #model_l=glm(Y_continuous ~ X1+X2, data_analysis)
  #patient_variance_fe=var(predict(model_l))
  
  ##method 2: PS method
  model_ps=multinom(Z ~ X1+X2, data = data_analysis,maxit=1000)
  predict_ps=fitted(model_ps)
  
  
  
  EY_ZX=NULL
  EY_ZX_FE=NULL
  VX=NULL
  VX_FE=NULL
  for (i in 1:max(data_analysis$Z))
  {
    Z_data=data_analysis
    Z_data$Z=rep(i,nrow(Z_data))
    ##by random effect model
    EY_ZX_temp=predict(model_r_e,Z_data,type="response")
    EY_ZX=cbind(EY_ZX,EY_ZX_temp)
    ##by fixed effect model
    EY_ZX_temp_fe=predict(model_f_e,Z_data,type="response")
    EY_ZX_FE=cbind(EY_ZX_FE,EY_ZX_temp_fe)
  }
  for (i in 1:nrow(EY_ZX))
  {
    VX[i]=sum(EY_ZX[i,]*predict_ps[i,])
    VX_FE[i]=sum(EY_ZX_FE[i,]*predict_ps[i,])
  }
  patient_variance_ps=var(VX)
  patient_variance_ps_fe=var(VX_FE)
  
  ##variance explained by patient hospital performance
  ##method 1
  ##random effect model
  #hospital_variance_re=as.data.frame(VarCorr(model_r_e))[1,4]
  
  ##method 2
  ##PS method
  EX=NULL
  EX_FE=NULL
  
  for (i in 1:nrow(EY_ZX))
  {
    EX[i]=sum((EY_ZX[i,]^2)*predict_ps[i,])-(sum(EY_ZX[i,]*predict_ps[i,]))^2
    EX_FE[i]=sum((EY_ZX_FE[i,]^2)*predict_ps[i,])-(sum(EY_ZX_FE[i,]*predict_ps[i,]))^2
  }
  hospital_variance_ps=mean(EX)
  hospital_variance_ps_fe=mean(EX_FE)
  Variance_Y=var(data_analysis$Y_binary)
  
  ##true_vairance_explained by hospital and patient
  True_patient_v_bin=var(data_analysis$True_v_patient_binary)
  True_hospital_v_bin=mean(data_analysis$True_v_hospital_binary)
  
  return(c(Variance_Y,hospital_variance_ps,hospital_variance_ps_fe,
           patient_variance_ps,patient_variance_ps_fe,
           True_patient_v_bin,True_hospital_v_bin))
}


############################################Data Generation##################################################
###number of hospitals=25
###varied number of pateints=500, 1000, 2500, 5000
set.seed(123)
data_test=list()
indicator=1
##Vary the number of hospital from 10 to 100
hospital_number=25
patient_number=c(500, 1000, 2500, 5000)
mu_Z_effect=0
sd_Z_effect=2
mu_Z_effects=rnorm(hospital_number,mean=mu_Z_effect,sd=sd_Z_effect)
hospital_generate_coefficient <- matrix(NA, hospital_number, 3)
hospital_generate_coefficient[,1] <- rnorm(hospital_number, mean=0, sd=0.25)
hospital_generate_coefficient[,2] <- rnorm(hospital_number, mean=0, sd=0.5)
hospital_generate_coefficient[,3] <- rnorm(hospital_number, mean=0, sd=0.5)
for(m in 1:length(patient_number))
  {
    ##1000 dataset for each criteria
  for (i in 1:1000)
    {
      ##data generation
      ##we generate 1000 dataset
    data_test[[indicator]] = datgen(n=patient_number[m],N=hospital_number,
                                      mu_Z_effects=mu_Z_effects,hospital_generate_coefficient=hospital_generate_coefficient,
                                      a=0,b=1,p=0.5,a0=0.0,betax1=1.0,betax2=2.0)
    indicator=indicator+1
    print(indicator)
    }
}


##save the data file
saveRDS(data_test, file = file.path(path, "data_fix_hospital.rds"))

##read the dataset file
data_test_fix_hospital=readRDS(file.path(path, "data_fix_hospital.rds"))

###number of patients=5000
###varied number of hospitals=10, 25, 50, 100
set.seed(123)
data_test=list()
indicator=1
hospital_number=c(10, 25, 50, 100)
patient_number=5000
mu_Z_effect=0
sd_Z_effect=2
for(m in 1:length(hospital_number))
{
  mu_Z_effects=rnorm(hospital_number[m],mean=mu_Z_effect,sd=sd_Z_effect)
  print(var(mu_Z_effects))
  hospital_generate_coefficient <- matrix(NA, hospital_number[m], 3)
  hospital_generate_coefficient[,1] <- rnorm(hospital_number[m], mean=0, sd=0.25)
  hospital_generate_coefficient[,2] <- rnorm(hospital_number[m], mean=0, sd=0.5)
  hospital_generate_coefficient[,3] <- rnorm(hospital_number[m], mean=0, sd=0.5)
  ##1000 dataset for each criteria
  for (i in 1:1000)
  {
    ##data generation
    ##we generate 1000 dataset
    data_test[[indicator]] = datgen(n=patient_number,N=hospital_number[m],
                                    mu_Z_effects=mu_Z_effects,hospital_generate_coefficient=hospital_generate_coefficient,
                                    a=0,b=1,p=0.5,a0=0.0,betax1=1.0,betax2=2.0)
    indicator=indicator+1
    # print(indicator)
  }
}


##save the data file
saveRDS(data_test, file=file.path(path, "data_fix_patient.rds"))

##read the dataset file
data_test_fix_patient=readRDS(file.path(path, "data_fix_patient.rds"))

##########################################Data Analysis#####################################################
##fix the number of hospitals
data_test=data_test_fix_hospital
rm(data_test_fix_hospital)
for(i in 1:length(data_test))
{
  print(i)
  result_continuous_outcome = analysis_continuous(data_test[[i]])
  write(result_continuous_outcome, file=file.path(path, "continuous_outcome_fix_hospital.txt"), ncolumns = 11, append=TRUE)
  result_binary_outcome = analysis_binary(data_test[[i]])
  write(result_binary_outcome, file=file.path(path, "binary_outcome_fix_hospital.txt"), ncolumns=7, append=TRUE)
}

##read-in first
result_continuous_outcome_fix_hospital=data.frame(continuous_outcome_fix_hospital)
names(result_continuous_outcome_fix_hospital)=c("var_Y_continuous","residual_var_fe_con","residual_var_ps_con",
                                   "hospital_var_re_con","hospital_var_ps_re_con",
                                   "hospital_var_ps_fe_con","patient_var_fe_con",
                                   "patient_var_ps_re_con","patient_var_ps_fe_con",
                                   "True_patient_v_con","True_hospital_v_con")

##read-in first
result_binary_outcome_fix_hospital=data.frame(binary_outcome_fix_hospital)
names(result_binary_outcome_fix_hospital)=c("var_Y_binary","hospital_var_ps_re_binary",
                               "hospital_var_ps_fe_binary",
                               "patient_var_ps_re_binary","patient_var_ps_fe_binary",
                               "True_patient_v_bin","True_hospital_v_bin")

result_fix_hospital=data.frame(result_continuous_outcome_fix_hospital,result_binary_outcome_fix_hospital)
#colMeans(result)
result_fix_hospital$N_hospital=rep(25,each=1000*4)
result_fix_hospital$N_patient=rep(c(500, 1000, 2500, 5000),each=1000)

##save the result
saveRDS(result_fix_hospital, file = file.path(path, "result_fix_hospital.rds"))

result_fix_hospital=readRDS(file.path(path, "result_fix_hospital.rds"))

##fix the number of patients
data_test=data_test_fix_patient
rm(data_test_fix_patient)
for(i in 1:length(data_test))
{
  print(i)
  result_continuous_outcome = analysis_continuous(data_test[[i]])
  write(result_continuous_outcome, file=file.path(path, "continuous_outcome_fix_patient.txt"), ncolumns = 11, append=TRUE)
  result_binary_outcome = analysis_binary(data_test[[i]])
  write(result_binary_outcome, file=file.path(path, "binary_outcome_fix_patient.txt"), ncolumns=7, append=TRUE)
}

##read-in first
result_continuous_outcome_fix_patient=data.frame(continuous_outcome_fix_patient)
names(result_continuous_outcome_fix_patient)=c("var_Y_continuous","residual_var_fe_con","residual_var_ps_con",
                                                "hospital_var_re_con","hospital_var_ps_re_con",
                                                "hospital_var_ps_fe_con","patient_var_fe_con",
                                                "patient_var_ps_re_con","patient_var_ps_fe_con",
                                                "True_patient_v_con","True_hospital_v_con")

##read-in first
result_binary_outcome_fix_patient=data.frame(binary_outcome_fix_patient)
names(result_binary_outcome_fix_patient)=c("var_Y_binary","hospital_var_ps_re_binary",
                                            "hospital_var_ps_fe_binary",
                                            "patient_var_ps_re_binary","patient_var_ps_fe_binary",
                                            "True_patient_v_bin","True_hospital_v_bin")

result_fix_patient=data.frame(result_continuous_outcome_fix_patient,result_binary_outcome_fix_patient)
#colMeans(result)
result_fix_patient$N_hospital=rep(c(10, 25, 50, 100),each=1000)
result_fix_patient$N_patient=rep(5000,each=1000*4)

##save the result
saveRDS(result_fix_patient, file = "result_fix_patient.rds")

##read the result file
result_fix_patient=readRDS(file.path(path, "result_fix_patient.rds"))

######################################Result Graphs######################################################

##manuscript Figure 2:

##barplot continuous case
bar_con=function(subdata)
{
  average=c(mean(subdata$hospital_var_ps_fe_con),mean(subdata$hospital_var_ps_re_con),
            mean(subdata$patient_var_ps_fe_con),mean(subdata$patient_var_ps_re_con),
            mean(subdata$residual_var_fe_con),mean(subdata$residual_var_ps_con))
  se=qnorm(0.975)*c(sd(subdata$hospital_var_ps_fe_con),sd(subdata$hospital_var_ps_re_con),
            sd(subdata$patient_var_ps_fe_con),sd(subdata$patient_var_ps_re_con),
            sd(subdata$residual_var_fe_con),sd(subdata$residual_var_ps_con))/sqrt(1000)
  se.lower=average-se
  se.upper=average+se
  lower=c(quantile(subdata$hospital_var_ps_fe_con,0.025),quantile(subdata$hospital_var_ps_re_con,0.025),
          quantile(subdata$patient_var_ps_fe_con,0.025),quantile(subdata$patient_var_ps_re_con,0.025),
          quantile(subdata$residual_var_fe_con,0.025),quantile(subdata$residual_var_ps_con,0.025))
  upper=c(quantile(subdata$hospital_var_ps_fe_con,0.975),quantile(subdata$hospital_var_ps_re_con,0.975),
          quantile(subdata$patient_var_ps_fe_con,0.975),quantile(subdata$patient_var_ps_re_con,0.975),
          quantile(subdata$residual_var_fe_con,0.975),quantile(subdata$residual_var_ps_con,0.975))
  true.value=rep(c(mean(subdata$True_hospital_v_con),mean(subdata$True_patient_v_con),pi^2/3),each=2)
  Method=rep(c("FE","RE"),3)
  variance_type=c("Hospital","Hospital","Patient","Patient","Residual","Residual")
  new=data.frame(average,lower,upper,Method,variance_type,true.value,se.lower,se.upper)
  
  bar=ggplot(new, aes(x=variance_type, y=average, fill=Method)) + 
    geom_bar(position=position_dodge(), stat="identity",
             colour="black",
             size=.3) +
    geom_errorbar(aes(ymin=lower, ymax=upper), # error bar with 95% quantile intervel
                  width=.6,                    # Width of the error bars
                  position=position_dodge(.9))+
    geom_errorbar(aes(ymin=se.lower, ymax=se.upper), # error bar with 95% CI
                  width=.3,                    # Width of the error bars
                  position=position_dodge(.9))+
    geom_point(aes(y=true.value,group=Method),
               stat="identity",
               size=0.8,
               position = position_dodge(width = .9))+
    ylim(0,5)+
    labs(x = "Components", y = "Variance Estimates")+
    scale_fill_manual(values=c("white","grey"))+
    theme_bw()
  
  return(bar)
}

#fix the number of patients=5000
subdata_h10_p5000=result_fix_patient[which(result_fix_patient$N_hospital==10), ]
subdata_h25_p5000=result_fix_patient[which(result_fix_patient$N_hospital==25), ]
subdata_h50_p5000=result_fix_patient[which(result_fix_patient$N_hospital==50), ]
subdata_h100_p5000=result_fix_patient[which(result_fix_patient$N_hospital==100), ]
plot1=bar_con(subdata_h10_p5000)+ggtitle("n=5000, m=10")
plot2=bar_con(subdata_h25_p5000)+ggtitle("n=5000, m=25")
plot3=bar_con(subdata_h50_p5000)+ggtitle("n=5000, m=50")
plot4=bar_con(subdata_h100_p5000)+ggtitle("n=5000, m=100")

#fix the number of hospitals=25
subdata_h25_p500=result_fix_hospital[which(result_fix_hospital$N_patient==500), ]
subdata_h25_p1000=result_fix_hospital[which(result_fix_hospital$N_patient==1000), ]
subdata_h25_p2500=result_fix_hospital[which(result_fix_hospital$N_patient==2500), ]
subdata_h25_p5000=result_fix_hospital[which(result_fix_hospital$N_patient==5000), ]
plot5=bar_con(subdata_h25_p500)+ggtitle("n=500, m=25")
plot6=bar_con(subdata_h25_p1000)+ggtitle("n=1000, m=25")
plot7=bar_con(subdata_h25_p2500)+ggtitle("n=2500, m=25")
plot8=bar_con(subdata_h25_p5000)+ggtitle("n=5000, m=25")

pdf(file="Plot1_con.pdf",width = 10, height = 12)
multiplot(plot1, plot2, plot3, plot4, plot5, plot6, plot7, plot8,cols=2)
dev.off()

##manuscript Figure 3:

plot_patient5000_h=ggplot(result_fix_patient, aes(hospital_var_ps_fe_con,colour=as.factor(N_hospital))) +
  geom_density(aes(linetype=as.factor(N_hospital)))+
  labs(title = "n=5000", x = "Estimated Hospital Variance", color = "m",linetype = "m")+
  scale_colour_grey()+
  theme_bw()+
  ylim(0,5)
plot_patient5000_p=ggplot(result_fix_patient, aes(patient_var_ps_fe_con,colour=as.factor(N_hospital))) +
  geom_density(aes(linetype=as.factor(N_hospital)))+
  labs(title = "n=5000", x = "Estimated Patient Variance", color = "m",linetype = "m")+
  scale_colour_grey()+
  theme_bw()+
  ylim(0,5)
plot_patient5000_h
plot_patient5000_p

plot_hospital25_h=ggplot(result_fix_hospital, aes(hospital_var_ps_fe_con,colour=as.factor(N_patient))) +
  geom_density(aes(linetype=as.factor(N_patient)))+
  labs(title = "m=25", x = "Estimated Hospital Variance", color = "n",linetype = "n")+
  scale_colour_grey()+
  theme_bw()+
  ylim(0,4)
plot_hospital25_p=ggplot(result_fix_hospital, aes(patient_var_ps_fe_con,colour=as.factor(N_patient))) +
  geom_density(aes(linetype=as.factor(N_patient)))+
  labs(title = "m=25", x = "Estimated Patient Variance", color = "n",linetype = "n")+
  scale_colour_grey()+
  theme_bw()+
  ylim(0,4)

plot_hospital25_h
plot_hospital25_p

pdf(file="Plot2_con.pdf",width = 10, height = 4)
multiplot(plot_hospital25_h, plot_hospital25_p,cols=2)
dev.off()

##manuscript Figure 4:

##barplot continuous case 
##(compare our estimator from fixed effect model and variance of random components from random effect mode)
bar_con2=function(subdata)
{
  average=c(mean(subdata$hospital_var_ps_fe_con),mean(subdata$hospital_var_re_con))
  se=1.96*c(sd(subdata$hospital_var_ps_fe_con),sd(subdata$hospital_var_re_con))/sqrt(1000)
  se.lower=average-se
  se.upper=average+se
  lower=c(quantile(subdata$hospital_var_ps_fe_con,0.025),quantile(subdata$hospital_var_re_con,0.025))
  upper=c(quantile(subdata$hospital_var_ps_fe_con,0.975),quantile(subdata$hospital_var_re_con,0.975))
  true.value=c(mean(subdata$True_hospital_v_con),mean(subdata$True_hospital_v_con))
  Method=c("FE","RE")
  variance_type=c("Hospital","Hospital")
  new=data.frame(average,lower,upper,Method,variance_type,true.value,se.lower,se.upper)
  
  bar=ggplot(new, aes(x=variance_type, y=average, fill=Method)) + 
    geom_bar(position=position_dodge(), stat="identity",
             colour="black",
             size=.3) +
    geom_errorbar(aes(ymin=lower, ymax=upper), # error bar with 95% quantile intervel
                  width=.6,                    # Width of the error bars
                  position=position_dodge(.9))+
    geom_errorbar(aes(ymin=se.lower, ymax=se.upper), # error bar with 95% CI
                  width=.3,                    # Width of the error bars
                  position=position_dodge(.9))+
    geom_point(aes(y=true.value,group=Method),
               stat="identity",
               size=0.8,
               position = position_dodge(width = .9))+
    labs(x = "Component", y = "Variance Estimates")+
    scale_fill_manual(values=c("white","grey"),
                      name="Method",
                      breaks=c("FE","RE"),
                      labels=c("FE",expression(hat(tau)^2)))+
    theme_bw()+ylim(0,5)
  
  return(bar)
}
#fix the number of patient=5000
subdata_h10_p5000=result_fix_patient[which(result_fix_patient$N_hospital==10), ]
subdata_h25_p5000=result_fix_patient[which(result_fix_patient$N_hospital==25), ]
subdata_h50_p5000=result_fix_patient[which(result_fix_patient$N_hospital==50), ]
subdata_h100_p5000=result_fix_patient[which(result_fix_patient$N_hospital==100), ]
plot1=bar_con2(subdata_h10_p5000)+ggtitle("n=5000, m=10")
plot2=bar_con2(subdata_h25_p5000)+ggtitle("n=5000, m=25")
plot3=bar_con2(subdata_h50_p5000)+ggtitle("n=5000, m=50")
plot4=bar_con2(subdata_h100_p5000)+ggtitle("n=5000, m=100")
#multiplot(plot1, plot2, plot3,plot4, cols=2)

#fix the number of hospital=25
subdata_h25_p500=result_fix_hospital[which(result_fix_hospital$N_patient==500), ]
subdata_h25_p1000=result_fix_hospital[which(result_fix_hospital$N_patient==1000), ]
subdata_h25_p2500=result_fix_hospital[which(result_fix_hospital$N_patient==2500), ]
subdata_h25_p5000=result_fix_hospital[which(result_fix_hospital$N_patient==5000), ]
plot5=bar_con2(subdata_h25_p500)+ggtitle("n=500, m=25")
plot6=bar_con2(subdata_h25_p1000)+ggtitle("n=1000, m=25")
plot7=bar_con2(subdata_h25_p2500)+ggtitle("n=2500, m=25")
plot8=bar_con2(subdata_h25_p5000)+ggtitle("n=5000, m=25")
#multiplot(plot5, plot6, plot7, plot8,cols=2)

pdf(file="Plot3_con.pdf", width = 7, height = 6)
multiplot(plot1,plot3,plot2,plot4,cols=2)
dev.off()

##############################################Binary Outcomes########################################################

##manuscript Figure 5:

bar_binary=function(subdata)
{
  average=c(mean(subdata$hospital_var_ps_fe_binary),mean(subdata$hospital_var_ps_re_binary),
            mean(subdata$patient_var_ps_fe_binary),mean(subdata$patient_var_ps_re_binary))
  se=qnorm(0.975)*c(sd(subdata$hospital_var_ps_fe_binary),sd(subdata$hospital_var_ps_re_binary),
            sd(subdata$patient_var_ps_fe_binary),sd(subdata$patient_var_ps_re_binary))/sqrt(1000)
  se.lower=average-se
  se.upper=average+se
  lower=c(quantile(subdata$hospital_var_ps_fe_binary,0.025),quantile(subdata$hospital_var_ps_re_binary,0.025),
          quantile(subdata$patient_var_ps_fe_binary,0.025),quantile(subdata$patient_var_ps_re_binary,0.025))
  upper=c(quantile(subdata$hospital_var_ps_fe_binary,0.975),quantile(subdata$hospital_var_ps_re_binary,0.975),
          quantile(subdata$patient_var_ps_fe_binary,0.975),quantile(subdata$patient_var_ps_re_binary,0.975))
  true.value=rep(c(mean(subdata$True_hospital_v_bin),mean(subdata$True_patient_v_bin)),each=2)
  Method=rep(c("FE","RE"),2)
  variance_type=c("Hospital","Hospital","Patient","Patient")
  new=data.frame(average,lower,upper,Method,variance_type,true.value,se.lower,se.upper)
  
  bar=ggplot(new, aes(x=variance_type, y=average, fill=Method)) + 
    geom_bar(position=position_dodge(), stat="identity",
             colour="black",
             size=.3) +
    geom_errorbar(aes(ymin=lower, ymax=upper), # error bar with 95% quantile intervel
                  width=.6,                    # Width of the error bars
                  position=position_dodge(.9))+
    geom_errorbar(aes(ymin=se.lower, ymax=se.upper), # error bar with 95% CI
                  width=.3,                    # Width of the error bars
                  position=position_dodge(.9))+
    geom_point(aes(y=true.value,group=Method),
               stat="identity",
               size=0.8,
               position = position_dodge(width = .9))+
    labs(x = "Components", y = "Variance Estimates")+
    scale_fill_manual(values=c("white","grey"))+
    theme_bw()+
    ylim(0,0.1)
  
  return(bar)
}

#fix the number of patients=5000
subdata_h10_p5000=result_fix_patient[which(result_fix_patient$N_hospital==10), ]
subdata_h25_p5000=result_fix_patient[which(result_fix_patient$N_hospital==25), ]
subdata_h50_p5000=result_fix_patient[which(result_fix_patient$N_hospital==50), ]
subdata_h100_p5000=result_fix_patient[which(result_fix_patient$N_hospital==100), ]
plot1=bar_binary(subdata_h10_p5000)+ggtitle("n=5000, m=10")
plot2=bar_binary(subdata_h25_p5000)+ggtitle("n=5000, m=25")
plot3=bar_binary(subdata_h50_p5000)+ggtitle("n=5000, m=50")
plot4=bar_binary(subdata_h100_p5000)+ggtitle("n=5000, m=100")

#fix the number of hospitals=25
subdata_h25_p500=result_fix_hospital[which(result_fix_hospital$N_patient==500), ]
subdata_h25_p1000=result_fix_hospital[which(result_fix_hospital$N_patient==1000), ]
subdata_h25_p2500=result_fix_hospital[which(result_fix_hospital$N_patient==2500), ]
subdata_h25_p5000=result_fix_hospital[which(result_fix_hospital$N_patient==5000), ]
plot5=bar_binary(subdata_h25_p500)+ggtitle("n=500, m=25")
plot6=bar_binary(subdata_h25_p1000)+ggtitle("n=1000, m=25")
plot7=bar_binary(subdata_h25_p2500)+ggtitle("n=2500, m=25")
plot8=bar_binary(subdata_h25_p5000)+ggtitle("n=5000, m=25")

pdf(file="Plot1_binary.pdf", width = 10, height = 12)
multiplot(plot1, plot2, plot3, plot4, plot5, plot6, plot7, plot8, cols=2)
dev.off()

##manuscript Figure 6:

plot_patient5000_h=ggplot(result_fix_patient, aes(hospital_var_ps_fe_binary,colour=as.factor(N_hospital))) +
  geom_density(aes(linetype=as.factor(N_hospital)))+
  labs(title = "n=5000", x = "Estimated Hospital Variance", color = "m",linetype = "m")+
  scale_colour_grey()+
  theme_bw()
plot_patient5000_p=ggplot(result_fix_patient, aes(patient_var_ps_fe_binary,colour=as.factor(N_hospital))) +
  geom_density(aes(linetype=as.factor(N_hospital)))+
  labs(title = "n=5000", x = "Estimated Patient Variance", color = "m",linetype = "m")+
  scale_colour_grey()+
  theme_bw()
plot_patient5000_h
plot_patient5000_p


plot_hospital25_h=ggplot(result_fix_hospital, aes(hospital_var_ps_fe_binary,colour=as.factor(N_patient))) +
  geom_density(aes(linetype=as.factor(N_patient)))+
  labs(title = "m=25", x = "Estimated Hospital Variance", color = "n",linetype = "n")+
  scale_colour_grey()+
  theme_bw()+
  ylim(0,180)
plot_hospital25_p=ggplot(result_fix_hospital, aes(patient_var_ps_fe_binary,colour=as.factor(N_patient))) +
  geom_density(aes(linetype=as.factor(N_patient)))+
  labs(title = "m=25", x = "Estimated Patient Variance", color = "n",linetype = "n")+
  scale_colour_grey()+
  theme_bw()+
  ylim(0,180)

plot_hospital25_h
plot_hospital25_p

pdf(file="Plot2_binary.pdf",width = 10, height = 4)
multiplot( plot_hospital25_h, plot_hospital25_p,cols=2)
dev.off()
